#include <windows.h>
#include "define.h" /* this include file stores typedef that can be easily used */

#ifdef __cplusplus
	extern "C" {
#endif


/** \defgroup PCI3120 xPCI-3120 functions
 * @{
 *		\addtogroup General xPCI-3120 general functions
 *		\ingroup PCI3120
 *		@{
 */

/**  Enables to get the number of xPCI-3120 boards present in the system. \n
 * The function fill the value pointed by pb_NbrOfBoards with the number of boards present.
 *                                                                            
 * @param[out] pb_NbrOfBoards Value that will be filled with the number of boards     
 * @return	0 : No error \n
 *		   -1 : Error while getting the number of boards \n
 */
_INT_ i_PCI3120_GetNumberOfBoards (PBYTE	pb_NbrOfBoards );


/** Get device information of the board.
 *                                                                   
 * @param[in] b_BoardIndex : Index of the board
 * @param[in] dw_IdentifierStringSize : size of the identifier parameter 
 * @param[out] pc_Identifier: Identifier of the board
 * @param[out] pdw_UINumber : This number is typically a user-perceived slot number, such as a number printed next to the slot on the board, or some other number that makes locating the physical device easier for the user. If the device is on a bus that has no UI number convention, or if the bus driver for the device cannot determine the UI number, this value is 0xFFFFFFFF.
 * @param[out] pdw_DeviceNumber : Device number.
 * @param[out] pdw_BusNumber : Bus number.
 * @param[out] pdw_BaseAddress0 : Base address 0 of the board
 * @param[out] pdw_BaseAddress1 : Base address 1 of the board
 * @param[out] pdw_BaseAddress2 : Base address 2 of the board
 * @param[out] pdw_Version : Version of the board (4 chars).
 * @param[out] pb_Interrupt : Interrupt line of the board
 * @return	 0 : No error \n
 *			-1 : Unable to open the board via index \n
 *			-2 : Error when calling driver's IOCTL \n
 *          -65534 : LoadLibrary error (can not load PCIe3121.DLL) \n 
 *			-65535 : Error function doesn't exist in the DLL (if PCIe3121 DLL wrapper is used)\n
 */
_INT_ i_PCI3120_GetBoardInformation(BYTE		b_BoardIndex,
									DWORD		dw_IdentifierStringSize,
									PCHAR		pc_Identifier,
									PDWORD		pdw_UINumber,
									PDWORD		pdw_DeviceNumber,
									PDWORD		pdw_BusNumber,
									PDWORD		pdw_BaseAddress0,
									PDWORD		pdw_BaseAddress1,
									PDWORD		pdw_BaseAddress2,
									PDWORD		pdw_Version,
									PBYTE		pb_Interrupt );
								

/** Open the board with the index : b_BoardIndex. A handle is returned to the \n    
 * user which allows to use the driver functions. \n 
 *                                                                                  
 * @param[in] b_BoardIndex : Index of the board to open                             
 * @param[out] ph_DeviceHandle : Handle of the selected xPCI-3120 board for using the functions
 * @return		0 : No error \n
 *             -1 : Unable to get the number of available boards \n                                    
 *             -2 : Wrong device index parameter. The parameter is too high \n
 *             -3 : Error while getting the device's class \n
 *             -4 : Error while getting informations about the device \n
 *             -5 : Memory allocation error \n
 *             -6 : Get device interface detail error \n
 *             -7 : Error while creating communication handle \n
 *			   -8 : Error in the initialization callback function \n
 *             -65534 : LoadLibrary error (can not load PCIe3121.DLL) \n 
 *			   -65535 : Error function doesn't exist in the DLL (if PCIe3121 DLL wrapper is used)\n
 */
_INT_ i_PCI3120_OpenBoardViaIndex (BYTE	b_BoardIndex,
								   PHANDLE	ph_DeviceHandle );  


/** Open the board with the indentifier : pc_Identifier. A handle is returned to the \n    
 * user which allows to use the following functions. \n                                                         
 *                                                                                  
 * @param[in] pc_Identifier : Identifier of the board to open
 * @param[out] ph_DeviceHandle : Handle of the board xPCI-3120 for using the functions
 * @return	 0 : No Error \n
			-1 : Unable to get the number of available boards \n  
			-2 : Unable to get the board identifier \n
			-3 : Unable to open the board \n
			-4 : Identifier not found \n
            -65534 : LoadLibrary error (can not load PCIe3121.DLL) \n 
			-65535 : Error function doesn't exist in the DLL (if PCIe3121 DLL wrapper is used)\n
 */
_INT_ i_PCI3120_OpenBoardViaIdentifier (PCHAR	pc_Identifier,
										PHANDLE	ph_DeviceHandle );


/** Release the handle of the board. Block the access to the board.
 *                                                                   
 * @param[in] h_DeviceHandle    : Handle of the board                
 * @return	 0 : No Error \n
 *			-1 : The device handle parameter is wrong \n
 *          -2 : Error occur by the close \n
 *			-65535 : Error function doesn't exist in the DLL (if PCIe3121 DLL wrapper is used)\n
 */
_INT_ i_PCI3120_CloseBoard (HANDLE	h_DeviceHandle );


/** Return the version of the DLL.
 *                                                                   
 * @param[out] pdw_Version  : Version. sample : 0x01010910 -> version 1.01 september 2010       
 * @return	0 : No Error \n
 *			-65535 : Error function doesn't exist in the DLL (if PCIe3121 DLL wrapper is used)\n
 */
_INT_ i_PCI3120_GetDLLVersion (PDWORD	pdw_Version);

/** Return the version of the device driver.
*                                                                   
* @param[in] h_DeviceHandle  : Handle of the board                
* @param[out] pdw_Version  : Version. sample : 0x01011211 -> version 1.01 - 12/2011
* @return    0 : No Error \n                                                     
*            -1 : Error while calling IOCTL function \n
*			 -65535 : Error function doesn't exist in the DLL (if PCIe3121 DLL wrapper is used)\n
*/
_INT_ i_PCI3120_GetDeviceDriverVersion(HANDLE h_DeviceHandle, PDWORD pdw_Version);

/**
 *		@}
 *		
 *		\addtogroup Interrupt xPCI-3120 interrupt management functions
 * 		\ingroup PCI3120
 *		@{
 */


/** Install an interrupt callback function.
 *	\n The interrupt callback is called after each hardware interrupt, and can access and process the data read during the interrupt
 * The value of b_InterruptMask will enable you to determine the source of the interruption:
 * <ul>
 *	<li>0x1: End of sequence when calling i_PCI3120_Read1AnalogInput or i_PCI3120_ReadMoreAnalogInputs. Value are available thanks to pw_Args.</li>
 *	<li>0x4: Internal FIFO overflow. If your computer is too slow to handle all the data, after a while you will have an internal data fifo overflow (dw_InterruptMask will be 0x4). 
			 You can change the size of the internal data fifo in the driver's configuration pages. </li>
 *	<li>0x8: DMA interrupt. You will receive this interrupt when using i_PCI3120_InitAndStartAnalogInputSequence. 
             When you get this interrupt, you have to read the data measured by the board by calling the function i_PCI3120_ReadSequenceData (see its own documentation).</li>
 *	<li>0x10: Timer interrupt.</li>
 *	<li>0x20: Watchdog interrupt.</li>
 * </ul>
 *
 * @param[in] h_DeviceHandle Handle of the board
 * @param[in] v_FunctionName Address of the callback function
 *	\li \b h_DeviceHandle : Handle of the board
 *	\li \b b_FIFOOverflow : Indicates the status of the kernel interrupt FIFO (0 : no overflow, 1 : overflow, loss of data)
 *	\li \b b_InterruptMask : Indicates the event that generated this interrupt
 *	\li \b pw_Args : Values read during the interrupt
 *	\li \b dw_ArgsCount : Number of values read during the interrupt
 *
 * @return	 0 : No Error \n
 *			-1 : The device handle parameter is wrong \n
 *			-2 : An interrupt routine was still isntalled. Remove it with i_PCI3120_ResetBoardIntRoutine and retry \n
 *			-3 : Error while creating kill event \n
 *			-4 : Error while creating interrupt event \n
 *			-5 : Error while creating interrupt thread \n
 *			-6 : Error while changing the priority class of the thread \n
 *			-7 : Error while changing the priority of the thread \n
 *			-8 : Error while calling the IOCTL command \n
 *			-9 : Error in the interrupt callback function \n
 *			-65535 : Error function doesn't exist in the DLL (if PCIe3121 DLL wrapper is used)\n
 */
_INT_ i_PCI3120_SetBoardIntRoutine (HANDLE	h_DeviceHandle,
									void	(*v_FunctionName)(	HANDLE	h_DeviceHandle,
																BYTE	b_FIFOOverflow,
																BYTE	b_InterruptMask,
																PWORD	pw_Args,
																DWORD	dw_ArgsCount )
);

/** Deinstall the interrupt callback function.
 *
 * @param[in] h_DeviceHandle    : Handle of the board                
 * @return	 0 : No Error
 *			-1 : The device handle parameter is wrong \n
 *			-2 : No interrupt routine installed \n
 *			-3 : Error in the reset interrupt callback function \n
 *			-65535 : Error function doesn't exist in the DLL (if PCIe3121 DLL wrapper is used)\n
 */
_INT_ i_PCI3120_ResetBoardIntRoutine (HANDLE	h_DeviceHandle );


/**
 *		@} 	
 *		\addtogroup IO xPCI-3120 Input/Output functions
 *		\ingroup PCI3120
 * 		@{ 	
 *			\addtogroup Digital xPCI-3120 digital I/O
 *			\ingroup IO
 * 			@{
 */


/** Enable the digital output memory \n
 * When digital output memory is enabled, the active channels stay active until you set them to unactive. \n
 * When digital output memory is disabled, the active channels are reset each time you activate another channel (with set 1 digital output on for example) \n.
 *                                                                          
 * @param[in] h_DeviceHandle Handle of the board
 *
 * @return	 0 : No Error \n                                             
 *			-1 : Error while calling IOCTL function (wrong handle ?) \n
 *			-65535 : Error function doesn't exist in the DLL (if PCIe3121 DLL wrapper is used)\n
 */
_INT_ i_PCI3120_SetDigitalOutputMemoryOn(HANDLE h_DeviceHandle );


/** Disable the digital output memory \n
 * When digital output memory is enabled, the active channels stay active until you set them to unactive. \n
 * When digital output memory is disabled, the active channels are reset each time you activate another channel (with set 1 digital output on for example) \n.
 *                                                                          
 * @param[in] h_DeviceHandle Handle of the board
 *
 * @return	 0 : No Error \n                                             
 *			-1 : 
 *			-65535 : Error function doesn't exist in the DLL (if PCIe3121 DLL wrapper is used)\n
 */
_INT_ i_PCI3120_SetDigitalOutputMemoryOff(HANDLE h_DeviceHandle );

/** Read 1 digital input of the board
 *                                                                          
 * @param[in] h_DeviceHandle Handle of the board                      
 * @param[in] b_Channel Channel to read (0 to 3)
 * @param[out] pb_Value Status of the digital input
 *
 * @return	 0 : No Error \n                                             
 *			-1 : Invalid channel \n           
 *			-2 : Error while calling IOCTL function (wrong handle ?) \n
 *			-65535 : Error function doesn't exist in the DLL (if PCIe3121 DLL wrapper is used)\n
 */
_INT_ i_PCI3120_Read1DigitalInput(HANDLE	h_DeviceHandle,
								  BYTE		b_Channel,
								  PBYTE		pb_Value );


/** Read the 4 digital inputs of the board
 *                                                                          
 * @param[in] h_DeviceHandle Handle of the board
 * @param[out] pb_Value Status of the digital inputs
 *
 * @return	 0 : No Error \n                                             
 *			-1 : Error while calling IOCTL function (wrong handle ?) \n
 *			-65535 : Error function doesn't exist in the DLL (if PCIe3121 DLL wrapper is used)\n
 */
_INT_ i_PCI3120_Read4DigitalInputs(HANDLE	h_DeviceHandle,
								   PBYTE	pb_Value );

/** Set 1 digital output of the board to HIGH
 *                                                                          
 * @param[in] h_DeviceHandle Handle of the board                      
 * @param[in] b_Channel Channel to set (0 to 3)
 *
 * @return	 0 : No Error \n                                             
 *			-1 : Wrong channel \n           
 *			-3 : Error while calling IOCTL function (wrong handle ?) \n
 *			-65535 : Error function doesn't exist in the DLL (if PCIe3121 DLL wrapper is used)\n
 */
_INT_ i_PCI3120_Set1DigitalOutputOn(HANDLE	h_DeviceHandle,
								  BYTE		b_Channel );


/** Set 1 digital output of the board to LOW
 *                                                                          
 * @param[in] h_DeviceHandle Handle of the board                      
 * @param[in] b_Channel Channel to reset (0 to 3)
 *
 * @return	 0 : No Error \n                                             
 *			-1 : Wrong channel \n          
 *			-3 : Error while calling IOCTL function (wrong handle ?) \n
 *			-65535 : Error function doesn't exist in the DLL (if PCIe3121 DLL wrapper is used)\n
 */
_INT_ i_PCI3120_Set1DigitalOutputOff(HANDLE	h_DeviceHandle,
									 BYTE	b_Channel );


/** Set the digital outputs set in the mask to HIGH
 *                                                                          
 * @param[in] h_DeviceHandle Handle of the board                      
 * @param[in] b_Mask Channels to set (0x0 to 0xF)
 *
 * @return	 0 : No Error \n                                             
 *			-1 : Wrong mask \n         
 *			-3 : Error while calling IOCTL function (wrong handle ?) \n
 *			-65535 : Error function doesn't exist in the DLL (if PCIe3121 DLL wrapper is used)\n
 */
_INT_ i_PCI3120_Set4DigitalOutputsOn(HANDLE	h_DeviceHandle,
									 BYTE	b_Mask );


/** Set the digital outputs set in the mask to LOW
 *                                                                          
 * @param[in] h_DeviceHandle Handle of the board                      
 * @param[in] b_Mask Channels to reset (0x0 to 0xF)
 *
 * @return	 0 : No Error \n                                             
 *			-1 : Wrong mask \n             
 *			-3 : Error while calling IOCTL function (wrong handle ?) \n
 *			-65535 : Error function doesn't exist in the DLL (if PCIe3121 DLL wrapper is used)\n
 */
_INT_ i_PCI3120_Set4DigitalOutputsOff(HANDLE	h_DeviceHandle,
									  BYTE	b_Mask );

/** Get the status of 1 digital output of the board
 *                                                                          
 * @param[in] h_DeviceHandle Handle of the board
 * @param[in] b_Channel Channel to get (0 to 3)
 * @param[out] pb_OutputsStatus Status of the digital output
 *
 * @return	 0 : No Error \n                                             
 *			-1 : Wrong channel \n
 *			-2 : Error while calling IOCTL function (wrong handle ?) \n
 *			-65535 : Error function doesn't exist in the DLL (if PCIe3121 DLL wrapper is used)\n
 */
_INT_ i_PCI3120_Get1DigitalOutputStatus(HANDLE	h_DeviceHandle,
										BYTE	b_Channel,
										PBYTE	pb_OutputsStatus );

/** Get the status of all the digital outputs of the board
 *                                                                          
 * @param[in] h_DeviceHandle Handle of the board
 * @param[out] pb_OutputsStatus Status of the digital output
 *
 * @return	 0 : No Error \n                                             
 *			-1 : Error while calling IOCTL function (wrong handle ?) \n
 *			-65535 : Error function doesn't exist in the DLL (if PCIe3121 DLL wrapper is used)\n
 */
_INT_ i_PCI3120_Get4DigitalOutputsStatus(HANDLE	h_DeviceHandle,
										 PBYTE	pb_OutputsStatus );


/** 		@}
 *			\addtogroup Analog xPCI-3120 analog I/O
 *			\ingroup IO
 * 			@{
 */

 /** Set analog input mode to single-ended or differential input \n
 * The input mode has to be set at least once before any call to an analog input function. \n
 * Once set, the configuration will last until the program ends, or another mode is set. \n
 * \n
 * In single-ended mode, 16 analog inputs are available. \n
 * In differential mode, there are only 8 analog inputs available. \n
 * Please refer to the manual for further information. \n
 *
 * @param[in] h_DeviceHandle Handle of the board
 * @param[in] b_InputMode Input mode (PCI3120_INPUT_MODE_SINGLE_ENDED or PCI3120_INPUT_MODE_DIFFERENTIAL)
 *
 * @return	 0 : No Error \n                                             
 *			-1 : The device handle parameter is wrong \n
 *			-2 : Invalid input mode \n
 *			-65535 : Error function doesn't exist in the DLL (if PCIe3121 DLL wrapper is used)\n
 */
_INT_ i_PCI3120_SetAnalogInputMode(HANDLE	h_DeviceHandle,
								   BYTE		b_InputMode );

 /** Get analog input mode (single-ended or differential input) \n
 * \n
 * In single-ended mode, 16 analog inputs are available. \n
 * In differential mode, there are only 8 analog inputs available. \n
 * Please refer to the manual for further information. \n
 *
 * @param[in] h_DeviceHandle Handle of the board
 * @param[out] pb_InputMode Input mode (PCI3120_INPUT_MODE_UNDEFINED, PCI3120_INPUT_MODE_SINGLE_ENDED or PCI3120_INPUT_MODE_DIFFERENTIAL)
 *
 * @return	 0 : No Error \n                                             
 *			-1 : The device handle parameter is wrong \n
 *			-65535 : Error function doesn't exist in the DLL (if PCIe3121 DLL wrapper is used)\n
 */
_INT_ i_PCI3120_GetAnalogInputMode(HANDLE	h_DeviceHandle,
								   PBYTE	pb_InputMode );


/** Reads the current value of the analog input channel b_Channel with a gain b_Gain, in the input voltage range b_Polarity and a conversion time w_ConvertingTime
 *
 * @param[in] h_DeviceHandle Handle of the board
 *
 * @param[in] b_Channel Channel to read (Single-ended : 0 to 15, differential : 0-7)
 *
 * @param[in] b_Gain Input gain. Adjusts the input voltage to be in the converter's range (-10V to 10V) and get the best resolution.
 *		\n	Gain acts like a multiplying coefficient, so a value of 2 will double the input voltage before it goes through the converter.
 *		\n	For example, if the input voltage is between 0V and 2V, a gain value of 5 should be used.
 *		\n	Possible values are PCI3120_GAIN_1, PCI3120_GAIN_2, PCI3120_GAIN_5 and PCI3120_GAIN_10
 *
 * @param[in] b_Polarity Selection of the input voltage range.
 *		\li	\b Unipolar : Input voltage between 0V and 10V (value APCI3120_UNIPOLAR)
 *		\li	\b Bipolar : Input voltage range between -10V and +10V. Twice the range, so half the resolution (value APCI3120_BIPOLAR)
 *		\n	N.B. : Those values are given for APCI3120_1_GAIN. For other values, see above
 *
 * @param[in] w_ConvertingTime Selection of the conversion time, from 10 to 32767us.
 *
 * @param[in] b_InterruptFlag Wether an interrupt is generated at EOS (End Of Sequence) or not
 *
 * @param[out] pw_ChannelValue The raw analog value (0x0000 to 0xFFFF)
 *
 * @return	 0 : No Error \n                                             
 *			-1 : The device handle parameter is wrong \n
 *			-3 : The input mode has not been set \n
 *			-4 : A channel index is invalid (single-ended mode) \n
 *			-5 : A channel index is invalid (differential mode) \n
 *			-6 : A gain value is wrong \n
 *			-7 : A polarity value is wrong \n
 *			-8 : The converting time is invalid \n
 *			-9 : The interrupt routine is not installed \n
 *			-10 : Error while calling IOCTL function (wrong handle ?) \n
 *			-110: The software trigger is invalid (if PCIe3121 DLL wrapper is used) \n
 *			-111: The external trigger is invalid (if PCIe3121 DLL wrapper is used) \n
 *			-65535 : Error function doesn't exist in the DLL (if PCIe3121 DLL wrapper is used)\n
 *
 */
_INT_ i_PCI3120_Read1AnalogInput (HANDLE	h_DeviceHandle,
								  BYTE		b_Channel,
								  BYTE		b_Gain,
								  BYTE		b_Polarity,
								  WORD		w_ConvertingTime,
								  BYTE		b_InterruptFlag,
								  PWORD		pw_ChannelValue );


/** Reads several analog input channels
 * \n The priority of the analog input channels is set with the scan list
 * \n The scan list allows to determine the input voltage range and the gain for each analog input channel
 * \n The gain is defined with parameter pb_Gain for each analog input channel
 * \n The input voltage range is defined with parameter pb_Polarity for each analog input channel
 *
 * @param[in] h_DeviceHandle Handle of the board
 * @param[in] b_ChannelCount Number of acquisitions in the sequence (1 to 16)
 * @param[in] pb_Channel Array of the channels to read
 * @param[in] pb_Gain Array of gain for each channel (see i_PCI3120_Read1AnalogInput)
 * @param[in] pb_Polarity Array of input voltage range for each channel (see i_PCI3120_Read1AnalogInput)
 * @param[in] w_ConvertingTime Selection of the conversion time, from 10us to 32767us.
 * @param[in] b_InterruptFlag Wether an interrupt is generated at EOS (End Of Sequence) or not.
 *		\n	Read values are available in the interrupt callback function (if PCIe3121 DLL wrapper is used)\n
 *
 * @param[out] pw_ChannelValue Raw output values (0x0 to 0xFFFF)
 *
 * @return	 0 : No Error \n                                             
 *			-1 : The device handle parameter is wrong \n
 *			-2 : The sequence size is wrong \n
 *			-3 : The input mode has not been set \n
 *			-4 : A channel index is invalid (single-ended mode) \n
 *			-5 : A channel index is invalid (differential mode) \n
 *			-6 : A gain value is wrong \n
 *			-7 : A polarity value is wrong \n
 *			-8 : The converting time is invalid \n
 *			-9 : The interrupt routine is not installed \n
 *			-10 : Error while calling IOCTL function (wrong handle ?) \n
 *			-110 : The software trigger is invalid (if PCIe3121 DLL wrapper is used) \n
 *			-111 : The external trigger is invalid (if PCIe3121 DLL wrapper is used) \n
 *			-1001 : Error while calling i_PCIe3121_TriggerAnalogAcquisition IOCTL function (wrong handle ?) \n
 *			-65535 : Error function doesn't exist in the DLL (if PCIe3121 DLL wrapper is used)\n
 */
_INT_ i_PCI3120_ReadMoreAnalogInputs (HANDLE	h_DeviceHandle,
									  BYTE		b_ChannelCount,
									  PBYTE		pb_Channel,
									  PBYTE		pb_Gain,
									  PBYTE		pb_Polarity,
									  WORD		w_ConvertingTime,
									  BYTE		b_InterruptFlag,
									  PWORD		pw_ChannelValue );


/** Write the value of one analog output of the board\n
 * In unipolar mode, output voltage range is 0V to 10V \n
 * In bipolar mode, output voltage range is -10V to 10V \n
 *
 * @param[in] h_DeviceHandle Handle of the board
 * @param[in] b_Channel Index of the analog output channel (0 to 7)
 * @param[in] b_Polarity Selection of the output voltage range (APCI3120_BIPOLAR or APCI3120_UNIPOLAR)
 * @param[in] w_Value Value to write to the output
 *		\li \b Unipolar : 0x0000 to 0x1FFF (0V to 10V)
 *		\li \b Bipolar : 0x0000 to 0x3FFF (-10V to 10V)
 *
 * @return	 0 : No Error \n                                             
 *			-1 : The device handle parameter is wrong \n
 *			-3 : Invalid channel \n
 *			-4 : Invalid polarity \n
 *			-5 : Invalid value in bipolar mode \n
 *			-6 : Invalid value in unipolar mode \n
 *			-7 : Error while calling IOCTL function (wrong handle ?) \n
 *			-65535 : Error function doesn't exist in the DLL (if PCIe3121 DLL wrapper is used)\n
 */
_INT_   i_PCI3120_Write1AnalogOutput (HANDLE	h_DeviceHandle,
									  BYTE		b_Channel,
									  BYTE		b_Polarity,
									  WORD		w_Value );


/** Write values of several analog outputs \n
 * In unipolar mode, output voltage range is 0V to 10V \n
 * In bipolar mode, output voltage range is -10V to 10V \n
 *
 * @param[in] h_DeviceHandle Handle of the board
 * @param[in] b_ChannelNbr Number of channels to write (1 to 8)
 * @param[in] pb_Channels Array of channels to write (0 to 7)
 * @param[in] pb_Polarity Array of polarity of the channel (APCI3120_BIPOLAR or APCI3120_UNIPOLAR)
 * @param[in] pw_Value Values to write to the output
 *
 * @return	 0 : No Error \n                                             
 *			-1 : The device handle parameter is wrong \n
 *			-2 : b_ChannelNbr is invalid \n
 *			-3 : Invalid channel \n
 *			-4 : Invalid plarity \n
 *			-5 : Invalid value in bipolar mode \n
 *			-6 : Invalid value in unipolar mode \n
 *			-7 : Error while calling IOCTL function (wrong handle ?) \n
 *  		-112 : both software and hardware trigger are enabled. (if PCIe3121 DLL wrapper is used). In that case, they must have the same type (one shot, sequence trigger or single sequence)
 *			-65535 : Error function doesn't exist in the DLL (if PCIe3121 DLL wrapper is used)\n
 */
_INT_ i_PCI3120_WriteMoreAnalogOutputs(HANDLE	h_DeviceHandle, 
									   BYTE		b_ChannelNbr, 
									   PBYTE	pb_Channels, 
									   PBYTE	pb_Polarity,
									   PWORD	pw_Value);

	
/** This function initializes a sequence acquisition
 * \n Thanks to a sequence acquisition, you can get analog values from the board very fastely (or slowly if you want)
 * \n After having initialized the sequence, to get the data you will have to wait an interrupt, and then read the data from the board.
 * \n You may have a look on the sample available with this DLL, that will explain you how to do.
 * \n
 * @param[in] h_DeviceHandle Handle of the board
 * @param[in] dw_SequenceCount The total number of sequence to acquire. Set it to 0 if you want to use continuous acquisition (i.e. infinite acquisition)
 * @param[in] dw_SequenceBeforeInterrupt Specify the number of sequence to wait before generating an interruption. For high frequency, set a high number (512 if using maximum frequency for example).
 * @param[in] b_ChannelCount The number of channels in one sequence. Rule: (b_ChannelCount * dw_SequenceBeforeInterrupt) must be a multiple of 2
 * @param[in] pb_Channel Array of the channels to read (see i_PCI3120_Read1AnalogInput)
 * @param[in] pb_Gain Array of gain for each channels (see i_PCI3120_Read1AnalogInput)
 * @param[in] pb_Polarity Array of polarity for each channels (see i_PCI3120_Read1AnalogInput)
 * @param[in] w_ConvertingTime Time interval in us between 2 conversions of successive input channels (from 10 us to 32767 us).
 * @param[in] b_DelayMode Specifiy the type of delay to use between two acquisitions of a sequence
 *		\li \b APCI3120_SIMPLE_MODUS : A conversion occurs every w_ConvertingTime (no delay)
 *		\li \b APCI3120_DELAY_MODUS : Conversions occur every w_ConvertingTime until all analog input channels have been acquired
 *			\n	Afterwards there is a waiting time of dw_DelayTime before the next cycle 
 *			\n	Please note: If the total time ( dw_DelayTime + b_ChannelCount * w_ConvertingTime ) is not a multiple of 50us,
 *          \n  then the resulting delay time is rounded because it depends on the timer resolution
 *          \n  One delay timer step corresponds to 50us
 *		\li \b APCI3120_DELAY_1_MODUS : Conversions occur every w_ConvertingTime until all analog input channels have been acquired
 *			\n	dw_DelayTime defines the time between the beginning of 2 cycles
 *			\n	That means dw_DelayTime can't be less than b_ChannelCount * w_ConvertingTime
 *			\n	This value must be a multiple of 50
 * @param[in] dw_DelayTime Waiting time in us between two conversion cycles (from 100 us to 3276850 us)
 *		\n	This parameter is only relevant if APCI3120_DELAY_MODUS or APCI3120_DELAY_1_MODUS are used
 *		\n	This parameter must be a multiple of 50
 * @param[in] b_ExtTriggerMode Two modes are possible :
 *		\li \b APCI3120_DISABLE No hardware trigger
 *		\li \b APCI3120_ENABLE Hardware trigger is set
 *			\n	A logic "1" on the digital input 1 starts the conversion. The conversion is stopped by calling "i_PCI3120_StopAnalogInputAcquisition"
 *
 * @return	  0 : No error
 *			 -1 : The device handle parameter is wrong \n
 *			 -2 : b_ChannelCount is invalid (must be between 1 and 16) \n
 *			 -3 : The current input mode is wrong. Please call i_PCI3120_SetAnalogInputMode before calling this function \n
 *			 -4 : The index of a channel in the pb_Channel array is too high \n
 *			 -5 : The gain of a channel in the pb_Gain array is invalid \n
 *			 -6 : The polarity of a channel in the pb_Polarity array is invalid \n
 *			 -7 : w_ConvertingTime is invalid (must be between 10 and 32767) \n
 *			 -8 : b_DelayMode is invalid \n
 *			 -9 : dw_DelayTime is invalid (must be between 100  and 3276850) \n
 *			-10 : dw_DelayTime is invalid (must be a multiple of 50) \n
 *			-11 : dw_DelayTime is not a multiple of 50 \n
 *			-12 : dw_DelayTime is smaller than (b_ChannelCount * w_ConvertingTime) \n
 *			-13 : no interrupt handler is installed. Please call i_PCI3120_SetBoardIntRoutine before calling this function \n
 *			-14 : dw_SequenceBeforeInterrupt is wrong. Must be higher than 1 \n
 *			-15 : (dw_SequenceBeforeInterrupt * b_ChannelCount ) is not a multiple of 2 \n
 *			-16 : b_ExtTriggerMode is invalid \n
 *			-17 : Error while calling IOCTL function (wrong handle ?) \n
 *			-65535 : Error function doesn't exist in the DLL (if PCIe3121 DLL wrapper is used)\n
 *
 *	Error -17: this error occurs when a driver error is detected. If you receive this error, the two mains grounds are:
 *	\li A wrong handle (h_DeviceHandle) is used. Did you open your board with i_PCI3120_OpenBoardViaIndex ?
 *  \li The parameter dw_SequenceBeforeInterrupt is too high. Check in the device manager, you will have a option DMA buffer size in the configuration of the board. Set the correct number and retry. Note: a channel is encoded on 2 bytes.
 */
_INT_ i_PCI3120_InitAndStartAnalogInputSequence(HANDLE	h_DeviceHandle,
												DWORD	dw_SequenceCount,
												DWORD	dw_SequenceBeforeInterrupt,
												BYTE	b_ChannelCount,
												PBYTE	pb_Channel,
												PBYTE	pb_Gain,
												PBYTE	pb_Polarity,
												WORD	w_ConvertingTime,
												BYTE	b_DelayMode,
												DWORD	dw_DelayTime,
												BYTE	b_ExtTriggerMode );

/** Stops any analog acquisition \n
 *
 * @param[in] h_DeviceHandle Handle of the board
 *
 * @return	 0 : No Error \n                                             
 *			-1 : Error while calling IOCTL function (wrong handle ?) \n
 *			-65535 : Error function doesn't exist in the DLL (if PCIe3121 DLL wrapper is used)\n
 */
_INT_ i_PCI3120_StopAnalogAcquisition(HANDLE	h_DeviceHandle );

/**
* Enables to read data acquired by a sequence acquisition. \n
* This function can be called in the interrupt routine that you registered. (see samples) \n
* @param deviceHandle handle to the board
* @param size size of data to read. To be powerfull, should be ( sequence size * sequence before interrupt ) * sizeof ( USHORT )
* @param data Pointer that will be fill with the data of the board
* @return	>= 0:	size of data read \n
*			-1	:	error while reading
*			-65535 : Error function doesn't exist in the DLL (if PCIe3121 DLL wrapper is used)\n
**/
_LONG_ l_PCI3120_ReadSequenceData(HANDLE deviceHandle,
						 DWORD size,
						 PVOID data);

/**			@}
 *		@}
 *		\addtogroup TimerWatchdog xPCI-3120 timer/watchdog functions
 *		\ingroup PCI3120
 *		@{
 */


/** Initializes and starts the timer / watchdog component
 *
 * @param[in] h_DeviceHandle Handle of the board
 * @param[in] b_TimerMode Defines the mode of the timer (APCI3120_TIMER or APCI3120_WATCHDOG)
 * @param[in] dw_DelayValue Time interval ( or watchdog time) of the timer / watchdog
 *		\n N.B. : The time value has to be a multiple of 50
 *		\li \b Timer : 100us to 838860850us (~838.86s)
 *		\li \b Watchdog : 50us to 838860850us (~838.86s)
 *
 * @param[in] b_InterruptFlag Enable or disable the interrupt at the end of a timer/watchdog count
 *		\li \b APCI3120_ENABLE :
 *			\n - Timer : an interrupt is generated at the end of each time interval
 *			\n - Watchdog : an interrupt is generated when the watchdog has run down
 *		\li \b APCI3120_DISABLE: No interrupt is generated
 *
 * @return	  0 : No error \n
 *			 -1 : The device handle parameter is wrong \n
 *			 -2 : The timer is already running \n
 *			 -3 : b_TimerMode is invalid \n
 *			 -4 : Timer mode choosen. dw_DelayValue is too low \n
 *			 -5 : Timer mode choosen. dw_DelayValue is too high \n
 *			 -6 : Watchdog mode choosen. dw_DelayValue is too low \n
 *			 -7 : Watchdog mode choosen. dw_DelayValue is too high \n
 *			 -8 : dw_DelayValue must be a multiple of 50 \n
 *			 -9 : b_InterruptFlag is APCI3120_ENABLE but no interrupt handler is installed. Please call i_PCI3120_SetBoardIntRoutine before calling this function \n
 *			-10 : Error while calling IOCTL function (wrong handle ?) \n
 *			-101 : b_TimerNb is invalid (if PCIe3121 DLL wrapper is used)\n
 *			-102 : b_TimerMode is invalid (if PCIe3121 DLL wrapper is used)\n
 *			-103 : b_TimeBase is invalid (if PCIe3121 DLL wrapper is used)\n
 *			-104 : b_OutputMode is invalid (if PCIe3121 DLL wrapper is used)\n
 *			-105 : b_InterruptEnabled is invalid (if PCIe3121 DLL wrapper is used)\n
 *			-106 : b_WatchdogEnabled is invalid (if PCIe3121 DLL wrapper is used)\n
 *			-107 : Error while calling IOCTL function (wrong handle ?)(if PCIe3121 DLL wrapper is used)\n
 *			-65535 : Error function doesn't exist in the DLL(if PCIe3121 DLL wrapper is used)\n
 */
_INT_ i_PCI3120_InitAndStartTimerWatchdog (HANDLE h_DeviceHandle,
											BYTE b_TimerMode,
											DWORD dw_DelayValue,
											BYTE b_InterruptFlag );

/** Stops the timer/watchdog
 *
 * @param[in] h_DeviceHandle Handle of the board
 *
 * @return	  0 : No error \n
 *			 -1 : The device handle parameter is wrong \n
 *			 -2 : The timer / watchdog is not running \n
 *			 -3 : Error while calling IOCTL function (wrong handle ?) \n
 *			 -101: b_TimerNb is invalid (if PCIe3121 DLL wrapper is used)\n
 *			 -65535 : Error function doesn't exist in the DLL(if PCIe3121 DLL wrapper is used)\n
 */
_INT_ i_PCI3120_StopTimerWatchdog (HANDLE h_DeviceHandle);

/** Reads the current value of the watchdog
 *
 * @param[in] h_DeviceHandle Handle of the board
 * @param[out] pb_WatchdogStatus Status of the watchdog (0: Watchdog is still running, 1: Watchdog has expired)
 *
 * @return	  0 : No error \n
 *			 -1 : The device handle parameter is wrong \n
 *			 -2 : The timer / watchdog is not running as a watchdog at the moment \n
 *			 -3 : Error while calling IOCTL function (wrong handle ?) \n
 *			 -65535 : Error function doesn't exist in the DLL(if PCIe3121 DLL wrapper is used)\n
 */
_INT_ i_PCI3120_ReadWatchdogStatus (HANDLE h_DeviceHandle, PBYTE pb_WatchdogStatus);


/** Reads the current value of the timer
 *
 * @param[in] h_DeviceHandle Handle of the board
 * @param[out] pdw_ReadValue Current timer value
 *
 * @return	  0 : No error \n
 *			 -1 : The device handle parameter is wrong \n
 *			 -2 : The timer / watchdog is not running as a timer at the moment \n
 *			 -3 : Error while calling IOCTL function (wrong handle ?) \n
 *			 -65535 : Error function doesn t exist in the DLL (if PCIe3121 DLL wrapper is used)\n
 */
_INT_ i_PCI3120_ReadTimerValue (HANDLE h_DeviceHandle, PDWORD pdw_ReadValue);

/** Writes a value to the timer
 *
 * @param[in] h_DeviceHandle Handle of the board
 * @param[out] dw_WriteValue Value to write
 *
 * @return	  0 : No error \n
 *			 -1 : The device handle parameter is wrong \n
 *			 -2 : The timer / watchdog is not running as a timer at the moment \n
 *			 -3 : Error while calling IOCTL function (wrong handle ?) \n
 *			-65535 : Error function doesn't exist in the DLL (if PCIe3121 DLL wrapper is used)\n
 */
_INT_ i_PCI3120_WriteTimerValue (HANDLE h_DeviceHandle, DWORD dw_WriteValue);

/**
 * @}
 * \ingroup General
 * @}
 */

#ifdef __cplusplus
	}
#endif
