using System;
using System.Collections.Generic;
using System.Text;
using System.Runtime.InteropServices;

namespace PCI3120_NET
{
    public class PCI3120
    {
        /// <summary>
        /// Store the count
        /// </summary>
        private static int boardCount;

        /// <summary>
        /// Store the index of the board
        /// </summary>
        private int index;

        /// <summary>
        /// Store an handle to the board. Will be used for interrupt routine for example
        /// </summary>
        private IntPtr handle;

        /// <summary>
        /// An object that will contain the timer event args
        /// Created only once, because the new command takes some time
        /// </summary>
        private TimerWatchdogEventArgs TimerWatchdogEventArgs;

        /// <summary>
        /// An object that will contain the analog input event args
        /// Created only once, because the new command takes some time
        /// </summary>
        private AnalogInputEventArgs analogInputEventArgs;

        /// <summary>
        /// An object that will contain the end of sequence event args
        /// Created only once, because the new command takes some time
        /// </summary>
        private EndOfSequenceEventArgs endOfSequenceEventArgs;

        /// <summary>
        /// Handler to the interrupt function of the board
        /// The function pointed by this delegate will be called each time the board interrupts
        /// </summary>
        private DllImport.InterruptFunctionEventHandler interruptFunctionHandler;

        #region "Event"

        /// <summary>
        /// The analog input event
        /// </summary>
        public event AnalogInputEventHandler AnalogInputEventReceived;

        /// <summary>
        /// Delegate to provide when an interrupt on the analog input occurs
        /// </summary>
        /// <param name="source"></param>
        /// <param name="args"></param>
        public delegate void AnalogInputEventHandler(object source, AnalogInputEventArgs args);

        /// <summary>
        /// The end of sequence event
        /// </summary>
        public event EndOfSequenceEventHandler EndOfSequenceEventReceived;

        /// <summary>
        /// Delegate to provide when an end of sequence interrupt occurs
        /// </summary>
        /// <param name="source"></param>
        /// <param name="args"></param>
        public delegate void EndOfSequenceEventHandler(object source, EndOfSequenceEventArgs args);

        /// <summary>
        /// The pld fifo overflow event
        /// </summary>
        public event PldFifoOverflowEventHandler PldFifoOverflowEventReceived;

        /// <summary>
        /// Delegate to provide when a PLD fifo overflow occurs
        /// </summary>
        /// <param name="source"></param>
        public delegate void PldFifoOverflowEventHandler(object source);

        /// <summary>
        /// Timer Watchdog event
        /// </summary>
        public event TimerWatchdogEventHandler TimerWatchdogEventReceived;

        /// <summary>
        /// Delegate to provide when Watchdog or Timer expired
        /// </summary>
        /// <param name="source"></param>
        /// <param name="args"></param>
        public delegate void TimerWatchdogEventHandler(object source, TimerWatchdogEventArgs args);


        #endregion

        #region "Constructor/Destructor"

        /// <summary>
        /// Static constructor to instanciate the class and initialize the variables
        /// </summary>
        static PCI3120()
        {
            PCI3120.boardCount = 0;

            // Get the number of boards
            if ((PCI3120.boardCount = GetBoardNumber()) == -1)
            {
                throw new Exception("PCI3120-GetBoardNumber error:" + PCI3120.boardCount.ToString());
            }
        }

        /// <summary>
        /// Constructor of the board
        /// Cannot be called; initialize the internal variables
        /// </summary>
        private PCI3120()
        {
            this.index = -1;
            this.handle = IntPtr.Zero;

            // Create the event args (once by board)
            this.TimerWatchdogEventArgs = new TimerWatchdogEventArgs();
            this.analogInputEventArgs = new AnalogInputEventArgs();
            this.endOfSequenceEventArgs = new EndOfSequenceEventArgs();

            // Create the interrupt handler (once by board)
            this.interruptFunctionHandler = new DllImport.InterruptFunctionEventHandler(InterruptFunction);

            // Tell the garbage collector to keep the pointer
            GC.KeepAlive(this.interruptFunctionHandler);
        }

        /// <summary>
        /// Destructor of the board
        /// Free all the ressources
        /// </summary>
        ~PCI3120()
        {
            DllImport.i_PCI3120_CloseBoard(handle);
            this.index = -1;
            this.handle = IntPtr.Zero;
        }

        #endregion

        #region "Utils"

        /// <summary>
        /// Enables to get an instance of a PCI3120 board
        /// </summary>
        /// <param name="index">The index of the board you want to open</param>
        /// <returns>null on error, instance of PCI3120 on success</returns>
        public static PCI3120 OpenBoard(int index)
        {
            PCI3120 boardInstance = null;
            IntPtr handle = IntPtr.Zero;

            // Check that the index is in the range
            if (index < 0 || index > PCI3120.boardCount)
            {
                Console.WriteLine("index not valid");
                return null;
            }

            // Try to open the board
            if (DllImport.i_PCI3120_OpenBoardViaIndex((byte)index, ref handle) != 0)
            {
                Console.WriteLine("i_PCI3120_OpenBoardViaIndex failed");
                return null;
            }

            // Create a new instance of the board
            boardInstance = new PCI3120();

            // Set its parameters
            boardInstance.index = index;
            boardInstance.handle = handle;

            // Return the instance
            return boardInstance;
        }

        /// <summary>
        /// Enables to get the number of PCI3120 boards plugged in the computer
        /// </summary>
        /// <returns>The number of board</returns>
        public static int GetBoardNumber()
        {
            int ret = 0;
            byte nbBoard = 0;

            // Call the DLL function
            ret = DllImport.i_PCI3120_GetNumberOfBoards(ref nbBoard);

            // Error case
            if (ret != 0) { return -1; }

             // Success
            else { return (int)nbBoard; }
        }

        /// <summary>
        /// Enables to signal to all the listeners that a PLD fifo overflow occurs
        /// </summary>
        /// <param name="eventArgs">The arguments of the event</param>
        protected virtual void OnPldFifoOverflowEventReceived()
        {
            if (this.PldFifoOverflowEventReceived != null)
            {
                this.PldFifoOverflowEventReceived(this);
            }
        }

        #endregion

        #region "Interrupt"

        /// <summary>
        /// Interrupt function - Called each time the board generates an interrupt
        /// </summary>
        /// <param name="handle"></param>
        /// <param name="overflow"></param>
        /// <param name="mask"></param>
        /// <param name="args"></param>
        /// <param name="argsSize"></param>
        private void InterruptFunction(IntPtr handle, byte overflow,byte mask,IntPtr args, uint argsSize )
        {
            // PLD fifo overflow ?
            if ((mask & 0x4) == 0x4)
            {
                this.OnPldFifoOverflowEventReceived();
            }
            // DMA ?
            else if ((mask & 0x8) == 0x8)
            {
                this.analogInputEventArgs.InterruptMask = mask;
                this.analogInputEventArgs.FifoOverflow = overflow;
                this.analogInputEventArgs.ArgsSize = argsSize;

                // Signal the event
                this.OnAnalogInputEventReceived(this.analogInputEventArgs);
            }
            // end of sequence ?
            else if ((mask & 1) == 1)
            {
                byte[] data = new byte[argsSize * sizeof(ushort)];

                this.endOfSequenceEventArgs.InterruptMask = mask;
                this.endOfSequenceEventArgs.FifoOverflow = overflow;
                this.endOfSequenceEventArgs.ArgsSize = argsSize;

                // Convert the args
                Marshal.Copy(args, data, 0, (int)argsSize * sizeof(ushort));
                for (int i = 0; i < argsSize; i++)
                {
                    this.endOfSequenceEventArgs.Args[i] = BitConverter.ToUInt16(data, i * sizeof(ushort));
                }

                // Signal the event
                this.OnEndOfSequenceEventReceived(this.endOfSequenceEventArgs);
            }
            //Timer 0x10 or watchdog 0x20 ?
            else if (((mask & 0x10) == 0x10 )|| ((mask & 0x20) == 0x20))
            {
                this.TimerWatchdogEventArgs.InterruptMask = mask;
                this.TimerWatchdogEventArgs.FifoOverflow = overflow;

                this.OnTimerWatchdogEventReceived(this.TimerWatchdogEventArgs);
            }
        }

        /// <summary>
        /// Set the board interrupt routine
        /// </summary>
        /// <returns>0 on success, != 0 on error</returns>
        public int SetBoardIntRoutine()
        {
            return (int)DllImport.i_PCI3120_SetBoardIntRoutine(this.handle, interruptFunctionHandler);
        }

        /// <summary>
        /// Reset the board interrupt routine
        /// </summary>
        /// <returns></returns>
        public int ResetBoardIntRoutine()
        {
            return (int)DllImport.i_PCI3120_ResetBoardIntRoutine(this.handle);
        }

        #endregion       

        #region "Analog inputs"

        /// <summary>
        /// Enables to signal to all the listeners that an event on the analog inputs occurs
        /// </summary>
        /// <param name="eventArgs">The arguments of the event</param>
        protected virtual void OnAnalogInputEventReceived(AnalogInputEventArgs eventArgs)
        {
            if (this.AnalogInputEventReceived != null)
            {
                this.AnalogInputEventReceived(this, eventArgs);
            }
        }

        /// <summary>
        /// Enables to signal to all the listeners that an end of sequence interrupt occurs
        /// </summary>
        /// <param name="eventArgs"></param>
        protected virtual void OnEndOfSequenceEventReceived(EndOfSequenceEventArgs eventArgs)
        {
            if (this.EndOfSequenceEventReceived != null)
            {
                this.EndOfSequenceEventReceived(this, eventArgs);
            }
        }

        /// <summary>
        /// Set the input mode of the board
        /// </summary>
        /// <param name="inputMode">1: single ended, 2: differential</param>
        /// <returns>0 on success, != 0 on error</returns>
        public int SetAnalogInputMode(byte inputMode)
        {
            return DllImport.i_PCI3120_SetAnalogInputMode(this.handle, inputMode);
        }

        /// <summary>
        /// Get the current input mode of the board
        /// </summary>
        /// <param name="inputMode">Will be filled with the current input mode. 1: single ended, 2: differential</param>
        /// <returns>0 on success, != 0 on error</returns>
        public int GetAnalogInputMode(ref byte inputMode)
        {
            return DllImport.i_PCI3120_GetAnalogInputMode(this.handle, ref inputMode);
        }

        /// <summary>
        /// Initialize and start a sequence acquisition
        /// </summary>
        /// <param name="sequenceCount">The number of sequence to acquire. Set to 0 if continuous acquisition</param>
        /// <param name="sequenceBeforeInterrupt">The number of sequence to be acquired per interrupt</param>
        /// <param name="channelCount">The number of channels in one sequence</param>
        /// <param name="channelArray">An array that contains the index of the channels that will be acquired</param>
        /// <param name="gainArray">An array that contains the gain of the channels</param>
        /// <param name="polarityArray">An array that contains the polarity of the channels</param>
        /// <param name="convertingTime">The converting time 5us to 65 535 000us</param>
        /// <param name="delayTime">Define the time between two sequences acquisition (5us to 65 535 000us)</param>
        /// <param name="softTriggerMode">Define the software trigger</param>
        /// <param name="extTriggerMode">Define the hardware trigger</param>
        /// <param name="sequenceTriggerCount">If trigger is enabled, and sequence mode is choosed, define the number of sequence that is acquired per trigger</param>
        /// <returns>0 on success, != 0 on error</returns>
        public int InitAndStartAnalogInputSequence(uint sequenceCount,
            uint sequenceBeforeInterrupt,
            byte channelCount,
            byte[] channelArray,
            byte[] gainArray,
            byte[] polarityArray,
            short convertingTime,
            byte delayMode,
            uint delayTime,
            byte extTriggerMode)
        {
            return DllImport.i_PCI3120_InitAndStartAnalogInputSequence(this.handle, sequenceCount, sequenceBeforeInterrupt, channelCount,
                channelArray, gainArray, polarityArray, convertingTime,delayMode, delayTime, extTriggerMode);
        }

        /// <summary>
        /// Stop any analog input acquisition that is running
        /// </summary>
        /// <returns>0 on success, != 0 on error</returns>
        public int StopAnalogAcquisition()
        {
            return DllImport.i_PCI3120_StopAnalogAcquisition(this.handle);
        }

        /// <summary>
        /// Enables to read the data from an analog input sequence
        /// </summary>
        /// <param name="size">Size of data to read</param>
        /// <param name="data">Pointer that will be filled with the read data</param>
        /// <returns> >= 0: number of data read, -1: error </returns>
        public int ReadSequenceData(uint size, ushort[] data)
        {
            return DllImport.l_PCI3120_ReadSequenceData(this.handle, size, data);
        }

        /// <summary>
        /// Enable to read more analog inputs
        /// </summary>
        /// <param name="channelCount">The number of channels to read</param>
        /// <param name="channelArray">An array that contains the index of the channels that will be acquired</param>
        /// <param name="gainArray">An array that contains the gain of the channels</param>
        /// <param name="polarityArray">An array that contains the polarity of the channels</param>
        /// <param name="convertingTime">The converting time 10us to 32767us</param>
        /// <param name="interruptFlag">0: no interrupt, 1: interrupt</param>
        /// <param name="value">An array that will be filled the values of the channels</param>
        /// <returns>0 on success, != 0 on error</returns>
        public int ReadMoreAnalogInputs(byte channelCount,
            byte[] channelArray,
            byte[] gainArray,
            byte[] polarityArray,
            ushort convertingTime,
            byte interruptFlag,
            ushort[] value)
        {
            return DllImport.i_PCI3120_ReadMoreAnalogInputs(this.handle, channelCount, channelArray, gainArray, polarityArray, convertingTime, interruptFlag, value);
        }

        #endregion

        #region "Analog outputs"

        /// <summary>
        /// Write one analog output
        /// </summary>
        /// <param name="channelIndex">Index of the channel to be written</param>
        /// <param name="polarity">0: bipolar, 1: unipolar</param>
        /// <param name="value">Value to be written</param>
        /// <returns>0 on success, != 0 on error</returns>
        public int Write1AnalogOutput(byte channelIndex,
            byte polarity,
            ushort value)
        {
            return DllImport.i_PCI3120_Write1AnalogOutput(this.handle, channelIndex, polarity, value);
        }

        /// <summary>
        /// Write one analog output
        /// </summary>
        /// <param name="channelIndex">Index of the channel to be written</param>
        /// <param name="polarity">0: bipolar, 1: unipolar</param>
        /// <param name="value">Value to be written</param>
        /// <returns>0 on success, != 0 on error</returns>
        public int WriteMoreAnalogOutputs(byte channelNbr,
            byte[] channelArray,
            byte[] polarityArray,
            ushort[] value)
        {
            return DllImport.i_PCI3120_WriteMoreAnalogOutputs(this.handle, channelNbr, channelArray, polarityArray, value);
        }




        #endregion

        #region "Digital inputs"

        /// <summary>
        /// Enables to read 4 digital inputs (per port)
        /// </summary>
        /// <param name="inputValues">A byte that will be filled with the current status of the inputs</param>
        /// <returns>0 on success, != 0 on error</returns>
        public int Read4DigitalInputs(ref byte inputValues)
        {
            // Call the function
            return DllImport.i_PCI3120_Read4DigitalInputs(this.handle, ref inputValues);
        }

        #endregion

        #region "Digital outputs"
        /// <summary>
        /// Enable the digital output memory 
        /// </summary>
        /// <returns>0 on success, != 0 on error</returns>
        public int SetDigitalOutputMemoryOn()
        {
            return DllImport.i_PCI3120_SetDigitalOutputMemoryOn(this.handle);
        }

        /// <summary>
        /// Disable the digital output memory 
        /// </summary>
        /// <returns>0 on success, != 0 on error</returns>
        public int SetDigitalOutputMemoryOff()
        {
            return DllImport.i_PCI3120_SetDigitalOutputMemoryOff(this.handle);
        }

        /// <summary>
        /// Enables to set several digital outputs to on (high state)
        /// </summary>
        /// <param name="value">Digital output mask value to set (0 to 15)</param>
        /// <returns>0 on success, != 0 on error</returns>
        public int Set4DigitalOutputsOn(byte value)
        {
            return (int)DllImport.i_PCI3120_Set4DigitalOutputsOn(this.handle, value);
        }

        /// <summary>
        /// Enables to set several digital outputs to off (low state)
        /// </summary>
        /// <param name="value">Digital output mask value to set (0 to 15)</param>
        /// <returns>0 on success, != 0 on error</returns>
        public int Set4DigitalOutputsOff(byte value)
        {
            return (int)DllImport.i_PCI3120_Set4DigitalOutputsOff(this.handle, value);
        }

        /// <summary>
        /// Read 4 digital outputs status
        /// </summary>
        /// <param name="value">Status of the digital outputs</param>
        /// <returns>0 on success, != 0 on error</returns>
        public int Get4DigitalOutputsStatus(ref byte outputStatus)
        {
            return (int)DllImport.i_PCI3120_Get4DigitalOutputsStatus(this.handle, ref outputStatus);
        }

        #endregion

        #region "TimerWatchdog"

        /// <summary>
        /// Enables to signal to all the listeners that an event on the analog inputs occurs
        /// </summary>
        /// <param name="eventArgs">The arguments of the event</param>
        protected virtual void OnTimerWatchdogEventReceived(TimerWatchdogEventArgs eventArgs)
        {
            if (this.TimerWatchdogEventReceived != null)
            {
                this.TimerWatchdogEventReceived(this, eventArgs);
            }
        }


        /// <summary>
        /// Init And Start Timer Watchdog
        /// </summary>
        /// <param name="timermode">Timer instead of watchdog</param>
        /// <param name="delayvalue">time interval of the timer (multiple of 50us)</param>
        /// <param name="interruptflag">enable/disable interrupt</param>
        /// <returns>0 on success, != 0 on error</returns>
        public int InitAndStartTimerWatchdog(byte timerMode, uint delayValue, byte interruptFlag)
        {
            return DllImport.i_PCI3120_InitAndStartTimerWatchdog(this.handle, timerMode, delayValue, interruptFlag);
        }

        /// <summary>
        /// Stop Timer Watchdg
        /// </summary>
        /// <returns>0 on success, != 0 on error</returns>
        public int StopTimerWatchdog()
        {
            return DllImport.i_PCI3120_StopTimerWatchdog(this.handle);
        }

        /// <summary>
        /// Read Watchdog Status
        /// </summary>
        /// <param name="watchdogstatus">0:running 1:expired</param>
        /// <returns>0 on success, != 0 on error</returns>
        public int ReadWatchdogStatus(ref byte outputStatus)
        {
            return DllImport.i_PCI3120_ReadWatchdogStatus(this.handle, ref outputStatus);
        }

        /// <summary>
        /// Read Timer Value
        /// </summary>
        /// <param name="readvalue">current timer value</param>
        /// <returns>0 on success, != 0 on error</returns>
        public int ReadTimerValue(ref uint value)
        {
            return DllImport.i_PCI3120_ReadTimerValue(this.handle,ref value);
        }

        /// <summary>
        /// Write Timer Value
        /// </summary>
        /// <param name="writevalue">value to write</param>
        /// <returns>0 on success, != 0 on error</returns>
        public int WriteTimerValue(uint value)
        {
            return DllImport.i_PCI3120_WriteTimerValue(this.handle, value);
        }

        #endregion



    }
}
