using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;

namespace WatchdogInterrupt
{
    public partial class WatchdogInterruptForm : Form
    {
        /// <summary>
        /// Handle to the first board
        /// </summary>
        PCI3120_NET.PCI3120 board;

       
        /// <summary>
        /// Some internal value. Store the number of interrupt per component
        /// </summary>
        private int watchdogInterruptNb = 0;

        private int watchdogCount = 0;

        /// <summary>
        /// Useful array
        /// </summary>
        private TrackBar[] outputTrackBar;

        /// <summary>
        /// Constructor of the form
        /// </summary>
        public WatchdogInterruptForm()
        {
            InitializeComponent();

            // Create the useful array
            this.outputTrackBar = new TrackBar[8];
            this.outputTrackBar[0] = this.outputTrack0;
            this.outputTrackBar[1] = this.outputTrack1;
            this.outputTrackBar[2] = this.outputTrack2;
            this.outputTrackBar[3] = this.outputTrack3;
            this.outputTrackBar[4] = this.outputTrack4;
            this.outputTrackBar[5] = this.outputTrack5;
            this.outputTrackBar[6] = this.outputTrack6;
            this.outputTrackBar[7] = this.outputTrack7;
        }

        /// <summary>
        /// Called at the first load
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void WatchdogInterruptForm_Load(object sender, EventArgs e)
        {
            int ret = 0;

            try
            {
                this.board = PCI3120_NET.PCI3120.OpenBoard(0);
                if (board == null)
                {
                    MessageBox.Show("Error while opening board 0");
                    Close();
                    return;
                }
            }
            catch (Exception)
            {
                MessageBox.Show("Unable to load the APCI-3120 driver DLL!");
                Close();
                return;
            }

            // Enable the interruption
            if ((ret = this.board.SetBoardIntRoutine()) != 0)
            {
                MessageBox.Show("Error " + ret.ToString() + " in SetBoardIntRoutine");
                Close();
                return;
            }

            // Stop the components. No check since if nothing is started the function will fail
            this.board.StopTimerWatchdog();

            // Initialize the analog outputs to 0V
            for (byte i = 0; i < 8; i++)
            {
                int returnValue = 0;
                // We use bipolar mode
                // 0x1fff is 0V
                // 0x0 is -10V
                // 0x3fff is 10V
                if ((returnValue = this.board.Write1AnalogOutput(i, 0, 0x1fff)) != 0)
                {
                    MessageBox.Show("Write1AnalogOutput error : " + returnValue.ToString());
                    Close();
                    return;
                }
            }

            // Listen to the timer watchdog interruption
            this.board.TimerWatchdogEventReceived += new PCI3120_NET.PCI3120.TimerWatchdogEventHandler(board_TimerWatchdogEventReceived);

            // Initialize graphics
            this.InitTrackBar();
        }

        /// <summary>
        /// Interrupt received
        /// </summary>
        /// <param name="source"></param>
        /// <param name="args"></param>
        void board_TimerWatchdogEventReceived(object source, PCI3120_NET.TimerWatchdogEventArgs args)
        {
                this.watchdogInterruptNb++;
                //this.ResetTrackBar();
        }

        /// <summary>
        /// Initialize the track bar
        /// </summary>
        private void InitTrackBar()
        {
            for (int i = 0; i < 8; i++)
            {
                this.outputTrackBar[i].Minimum = -10;
                this.outputTrackBar[i].Maximum = 10;
                this.outputTrackBar[i].TickFrequency = 1;
                this.outputTrackBar[i].SmallChange = 1;
                this.outputTrackBar[i].LargeChange = 2;
                this.outputTrackBar[i].ValueChanged += new EventHandler(WatchdogInterruptForm_ValueChanged);
            }
        }


        /// <summary>
        /// put trackbar to 0 after watchdog rundown
        /// </summary>
        private void ResetTrackBar()
        {
            for (int i = 0; i < 8; i++)
                this.outputTrackBar[i].Value = 0;
        }

        /// <summary>
        /// Compute the value that must be written in the board
        /// </summary>
        /// <param name="value">The track bar value: -10 to +10</param>
        /// <returns></returns>
        private ushort ComputeOutputValue(int value)
        {
            value += 10;
            return (ushort)((value * 0x3FFF) / 20);
        }

        /// <summary>
        /// Analog output value is changed
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        void WatchdogInterruptForm_ValueChanged(object sender, EventArgs e)
        {
            for (byte i = 0; i < 8; i++)
            {
                if (sender == this.outputTrackBar[i])
                {
                    int returnValue = 0;
                    if ((returnValue = this.board.Write1AnalogOutput(i, 0, ComputeOutputValue(this.outputTrackBar[i].Value))) != 0)
                    {
                        MessageBox.Show("Write1AnalogOutput error : " + returnValue.ToString());
                        Close();
                    }
                    return;
                }
            }
        }      

        /// <summary>
        /// Occurs while the form is closing
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void WatchdogInterruptForm_FormClosing(object sender, FormClosingEventArgs e)
        {
            int ret = 0;

            this.board.StopTimerWatchdog();

            
            // Disable the interruption
            if ((ret = this.board.ResetBoardIntRoutine()) != 0)
            {
                MessageBox.Show("Error " + ret.ToString() + " in ResetBoardIntRoutine");
                return;
            }
        }

        /// <summary>
        /// Start watchdog
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void watchdogStart_Click(object sender, EventArgs e)
        {
            int ret = 0;
            this.CHB_interrupt.Enabled = false;
            this.NUD_DelayValue.Enabled = false;

            byte interrupt = CHB_interrupt.Checked ? (byte)1 : (byte)0;
            NUD_DelayValue.Value = ((((uint)NUD_DelayValue.Value + 25) / 50) * 50);
            // Initialize the component as a watchdog, 1 second default reload time, enable interruptions
            if ((ret = this.board.InitAndStartTimerWatchdog(61, (uint)NUD_DelayValue.Value, interrupt)) != 0)
            {
                MessageBox.Show("Error " + ret.ToString() + " in InitAndStartTimerWatchdog");
                return;
            }

            // Disable/enable button
            this.watchdogStart.Enabled = false;
            this.watchdogStop.Enabled = true;
            // Start the display timer
            watchdogDisplayTimer.Start();
        }

        /// <summary>
        /// Display timer ticks for watchdog
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void watchdogDisplayTimer_Tick(object sender, EventArgs e)
        {
            int ret = 0;
            byte status = 0;


            if (CHB_interrupt.Checked)
            {
                watchdogValue.Text = this.watchdogInterruptNb.ToString();
            }
            else
            {
                try
                {
                    if ((ret = this.board.ReadWatchdogStatus(ref status)) != 0)
                    {
                        MessageBox.Show("Error " + ret.ToString() + " in ReadTimerWatchdogValue");
                        Close();
                        return;
                    }
                }
                catch (Exception)
                {
                    MessageBox.Show("problem reading the watchdog status");
                    Close();
                    return;
                }

                if ((status & 1) != 0)
                {
                    this.watchdogCount++;
                    watchdogValue.Text = this.watchdogCount.ToString();
                    //this.ResetTrackBar();
                }
            }
        }


        /// <summary>
        /// Stop watchdog
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void watchdogStop_Click(object sender, EventArgs e)
        {
            int ret = 0;

            // Stop the timer
            if ((ret = this.board.StopTimerWatchdog()) != 0 )
            {
                MessageBox.Show("Error " + ret.ToString() + " in StopTimerWatchdog");
                return;
            }
            // Disable/enable button
            this.CHB_interrupt.Enabled = true;
            this.watchdogStart.Enabled = true;
            this.watchdogStop.Enabled = false;
            this.NUD_DelayValue.Enabled = true;
            // Stop the display timer
            watchdogDisplayTimer.Stop();
        }

        private void CHB_interrupt_CheckedChanged(object sender, EventArgs e)
        {
          counter1ValueLabel.Text = (CHB_interrupt.Checked ? "Interruptions count" : "watchdog rundown count").ToString();
          watchdogValue.Text = (CHB_interrupt.Checked ? this.watchdogInterruptNb : this.watchdogCount).ToString();
        }

    }
}